@@ -20,10 +20,7 @@
    Author: Bastien Nocera <hadess@hadess.net>
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 
 #ifndef TOTEM_PL_PARSER_MINI
 #include <string.h>
