@@ -18,9 +18,81 @@
    Boston, MA 02111-1307, USA.
 
    Authors: Bastien Nocera <hadess@hadess.net>
-
  */
 
+/**
+ * SECTION:totem-pl-parser
+ * @short_description: playlist parser
+ * @stability: Stable
+ * @include: totem-pl-parser.h
+ *
+ * #TotemPlParser is a general-purpose playlist parser and writer, with
+ * support for several different types of playlist.
+ *
+ * <example>
+ *  <title>Reading a Playlist</title>
+ *  <programlisting>
+ * TotemPlParser *pl = totem_pl_parser_new ();
+ * g_object_set (pl, "recurse", FALSE, "disable-unsafe", TRUE, NULL);
+ * g_signal_connect (G_OBJECT (pl), "playlist-started", G_CALLBACK (playlist_started), NULL);
+ * g_signal_connect (G_OBJECT (pl), "entry-parsed", G_CALLBACK (entry_parsed), NULL);
+ *
+ * if (totem_pl_parser_parse (pl, "http://example.com/playlist.pls", FALSE) != TOTEM_PL_PARSER_RESULT_SUCCESS)
+ * 	g_error ("Playlist parsing failed.");
+ *
+ * g_object_unref (pl);
+ *  </programlisting>
+ * </example>
+ *
+ * <example>
+ *  <title>Getting Metadata from Entries</title>
+ *  <programlisting>
+ * static void
+ * entry_parsed (TotemPlParser *parser, const gchar *uri, GHashTable *metadata, gpointer user_data)
+ * {
+ * 	gchar *title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
+ * 	if (title != NULL)
+ * 		g_message ("Entry title: %s", title);
+ * 	else
+ * 		g_message ("Entry (URI: %s) has no title.", uri);
+ * }
+ *  </programlisting>
+ * </example>
+ *
+ * <example>
+ *  <title>Writing a Playlist</title>
+ *  <programlisting>
+ * void
+ * parser_func (GtkTreeModel *model, GtkTreeIter *iter, gchar **uri, gchar **title, gboolean *custom_title, gpointer user_data)
+ * {
+ * 	gtk_tree_model_get (model, iter,
+ * 		0, uri,
+ * 		1, title,
+ * 		2, custom_title,
+ * 		-1);
+ * }
+ *
+ * {
+ * 	TotemPlParser *pl;
+ * 	GtkTreeModel *tree_model;
+ *
+ * 	pl = totem_pl_parser_new ();
+ *
+ * 	&slash;* Your tree model can be as simple or as complex as you like;
+ * 	 * parser_func() will just have to return the entry title, URI and custom title flag from it. *&slash;
+ * 	tree_model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+ * 	populate_model (tree_model);
+ *
+ * 	if (totem_pl_parser_write (pl, tree_model, parser_func, "/tmp/playlist.pls", TOTEM_PL_PARSER_PLS, NULL, NULL) != TRUE)
+ * 		g_error ("Playlist writing failed.");
+ *
+ * 	g_object_unref (tree_model);
+ * 	g_object_unref (pl);
+ * }
+ *  </programlisting>
+ * </example>
+ **/
+
 #include "config.h"
 
 #include <string.h>
@@ -109,26 +181,62 @@ enum {
 };
 
 static int totem_pl_parser_table_signals[LAST_SIGNAL];
+static GParamSpecPool *totem_pl_parser_pspec_pool = NULL;
 static gboolean i18n_done = FALSE;
 
-static void totem_pl_parser_class_init (TotemPlParserClass *class);
-static void totem_pl_parser_init       (TotemPlParser      *parser);
+static void totem_pl_parser_class_init (TotemPlParserClass *klass);
+static void totem_pl_parser_base_class_finalize	(TotemPlParserClass *klass);
+static void totem_pl_parser_init       (TotemPlParser *parser);
 static void totem_pl_parser_finalize   (GObject *object);
 
-G_DEFINE_TYPE(TotemPlParser, totem_pl_parser, G_TYPE_OBJECT)
+static void totem_pl_parser_init       (TotemPlParser      *self);
+static void totem_pl_parser_class_init (TotemPlParserClass *klass);
+static gpointer totem_pl_parser_parent_class = NULL;
+
+GType
+totem_pl_parser_get_type (void)
+{
+	static volatile gsize g_define_type_id__volatile = 0;
+	if (g_once_init_enter (&g_define_type_id__volatile))
+	{
+		const GTypeInfo g_define_type_info = {
+			sizeof (TotemPlParserClass),
+			NULL,
+			(GBaseFinalizeFunc) totem_pl_parser_base_class_finalize,
+			(GClassInitFunc) totem_pl_parser_class_init,
+			NULL,
+			NULL,
+			sizeof (TotemPlParser),
+			0,
+			(GInstanceInitFunc) totem_pl_parser_init,
+		};
+		GType g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "TotemPlParser", &g_define_type_info, 0); 
+		g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+	} 
+	return g_define_type_id__volatile; 
+}
 
 static void
 totem_pl_parser_class_init (TotemPlParserClass *klass)
 {
+	GParamSpec *pspec;
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	totem_pl_parser_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (TotemPlParserPrivate));
 
 	object_class->finalize = totem_pl_parser_finalize;
 	object_class->set_property = totem_pl_parser_set_property;
 	object_class->get_property = totem_pl_parser_get_property;
 
-	/* properties */
+	/* Properties */
+
+	/**
+	 * TotemPlParser:recurse:
+	 *
+	 * If %TRUE, the parser will recursively fetch playlists linked to by
+	 * the current one.
+	 **/
 	g_object_class_install_property (object_class,
 					 PROP_RECURSE,
 					 g_param_spec_boolean ("recurse",
@@ -137,6 +245,11 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+	/**
+	 * TotemPlParser:debug:
+	 *
+	 * If %TRUE, the parser will output debug information.
+	 **/
 	g_object_class_install_property (object_class,
 					 PROP_DEBUG,
 					 g_param_spec_boolean ("debug",
@@ -145,6 +258,12 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * TotemPlParser:force:
+	 *
+	 * If %TRUE, the parser will attempt to parse a playlist, even if it
+	 * appears to be unsupported (usually because of its filename extension).
+	 **/
 	g_object_class_install_property (object_class,
 					 PROP_FORCE,
 					 g_param_spec_boolean ("force",
@@ -153,6 +272,13 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * TotemPlParser:disable-unsafe:
+	 *
+	 * If %TRUE, the parser will not parse unsafe locations, such as local devices
+	 * and local files if the playlist isn't local. This is useful if the library
+	 * is parsing a playlist from a remote location such as a website.
+	 **/
 	g_object_class_install_property (object_class,
 					 PROP_DISABLE_UNSAFE,
 					 g_param_spec_boolean ("disable-unsafe",
@@ -161,7 +287,14 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
-	/* Signals */
+	/**
+	 * TotemPlParser::entry-parsed:
+	 * @parser: the object which received the signal
+	 * @uri: the URI of the entry parsed
+	 * @metadata: a #GHashTable of metadata relating to the entry added
+	 *
+	 * The ::entry-parsed signal is emitted when a new entry is parsed.
+	 */
 	totem_pl_parser_table_signals[ENTRY_PARSED] =
 		g_signal_new ("entry-parsed",
 			      G_TYPE_FROM_CLASS (klass),
@@ -170,6 +303,18 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      NULL, NULL,
 			      totemplparser_marshal_VOID__STRING_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
+	/**
+	 * TotemPlParser::playlist-started:
+	 * @parser: the object which received the signal
+	 * @uri: the URI of the new playlist started
+	 * @metadata: a #GHashTable of metadata relating to the playlist that
+	 * started.
+	 *
+	 * The ::playlist-started signal is emitted when a playlist parsing has
+	 * started. This signal isn't emitted for all types of playlists, but
+	 * can be relied on to be called for playlists which support playlist
+	 * metadata, such as title.
+	 */
 	totem_pl_parser_table_signals[PLAYLIST_STARTED] =
 		g_signal_new ("playlist-started",
 			      G_TYPE_FROM_CLASS (klass),
@@ -178,6 +323,15 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      NULL, NULL,
 			      totemplparser_marshal_VOID__STRING_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
+	/**
+	 * TotemPlParser::playlist-ended:
+	 * @parser: the object which received the signal
+	 * @uri: the URI of the playlist that finished parsing.
+	 *
+	 * The ::playlist-ended signal is emitted when a playlist is finished
+	 * parsing. It is only called when #TotemPlParser::playlist-started
+	 * has been called for that playlist.
+	 */
 	totem_pl_parser_table_signals[PLAYLIST_ENDED] =
 		g_signal_new ("playlist-ended",
 			      G_TYPE_FROM_CLASS (klass),
@@ -186,6 +340,113 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1, G_TYPE_STRING);
+
+	/* param specs */
+	totem_pl_parser_pspec_pool = g_param_spec_pool_new (FALSE);
+	pspec = g_param_spec_string ("url", "url",
+				     "URL to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("title", "title",
+				     "Title of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("author", "author",
+				     "Author of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("genre", "genre",
+				     "Genre of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("base", "base",
+				     "Base URL of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("volume", "volume",
+				     "Default playback volume (in percents)", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("autoplay", "autoplay",
+				     "Whether or not to autoplay the stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("duration", "duration",
+				     "String representing the duration of the entry, used for still images", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("starttime", "starttime",
+				     "String representing the start time of the stream (initial seek)", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("copyright", "copyright",
+				     "Copyright of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("abstract", "abstract",
+				     "Abstract of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("moreinfo", "moreinfo",
+				     "URL to get more information for item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("screensize", "screensize",
+				     "String representing the default movie size (double, full or original)", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("ui-mode", "ui-mode",
+				     "String representing the default UI mode (only compact is supported)", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("endtime", "endtime",
+				     "String representing the end time of the stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_boolean ("is-playlist", "is-playlist",
+				      "Boolean saying whether the entry pushed is the top-level of a playlist", FALSE,
+				      G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("description", "description",
+				     "String representing the description of the stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("publication-date", "publication-date",
+				     "String representing the publication date of the stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("filesize", "filesize",
+				     "String representing the filesize of a file", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("language", "language",
+				     "String representing the language of a stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("contact", "contact",
+				     "String representing the contact for a playlist", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("image-url", "image-url",
+				     "String representing the location of an image for a playlist", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+}
+
+static void
+totem_pl_parser_base_class_finalize (TotemPlParserClass *klass)
+{
+	GList *list, *node;
+
+	list = g_param_spec_pool_list_owned (totem_pl_parser_pspec_pool, G_OBJECT_CLASS_TYPE (klass));
+	for (node = list; node; node = node->next)
+	{
+		GParamSpec *pspec = node->data;
+
+		g_param_spec_pool_remove (totem_pl_parser_pspec_pool, pspec);
+		g_param_spec_unref (pspec);
+	}
+	g_list_free (list);
 }
 
 static void
@@ -265,6 +526,13 @@ totem_pl_parser_init_i18n (void)
 	}
 }
 
+/**
+ * totem_pl_parser_new:
+ *
+ * Creates a #TotemPlParser object.
+ *
+ * Return value: a new #TotemPlParser
+ */
 TotemPlParser *
 totem_pl_parser_new (void)
 {
@@ -272,6 +540,14 @@ totem_pl_parser_new (void)
 	return TOTEM_PL_PARSER (g_object_new (TOTEM_TYPE_PL_PARSER, NULL));
 }
 
+/**
+ * totem_pl_parser_playlist_end:
+ * @parser: a #TotemPlParser
+ * @playlist_uri: the playlist URI
+ *
+ * Emits the #TotemPlParser::playlist-ended signal on @parser for
+ * the playlist @playlist_uri.
+ **/
 void
 totem_pl_parser_playlist_end (TotemPlParser *parser, const char *playlist_uri)
 {
@@ -373,6 +649,14 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 	return g_strdup (mimetype);
 }
 
+/**
+ * totem_pl_parser_base_url:
+ * @url: a URI
+ *
+ * Returns the parent URI of @url.
+ *
+ * Return value: a newly-allocated string containing @url's parent URI, or %NULL
+ **/
 char *
 totem_pl_parser_base_url (const char *url)
 {
@@ -398,6 +682,15 @@ totem_pl_parser_base_url (const char *url)
 	return base;
 }
 
+/**
+ * totem_pl_parser_line_is_empty:
+ * @line: a playlist line to check
+ *
+ * Checks to see if the given string line is empty or %NULL,
+ * counting tabs and spaces, but not newlines, as "empty".
+ *
+ * Return value: %TRUE if @line is empty
+ **/
 gboolean
 totem_pl_parser_line_is_empty (const char *line)
 {
@@ -413,6 +706,16 @@ totem_pl_parser_line_is_empty (const char *line)
 	return TRUE;
 }
 
+/**
+ * totem_pl_parser_write_string:
+ * @handle: a #GnomeVFSHandle to an open file
+ * @buf: the string buffer to write out
+ * @error: return location for a #GError, or %NULL
+ *
+ * Writes the string @buf out to the file specified by @handle.
+ *
+ * Return value: %TRUE on success
+ **/
 gboolean
 totem_pl_parser_write_string (GnomeVFSHandle *handle, const char *buf, GError **error)
 {
@@ -422,6 +725,17 @@ totem_pl_parser_write_string (GnomeVFSHandle *handle, const char *buf, GError **
 	return totem_pl_parser_write_buffer (handle, buf, len, error);
 }
 
+/**
+ * totem_pl_parser_write_buffer:
+ * @handle: a #GnomeVFSHandle to an open file
+ * @buf: the string buffer to write out
+ * @len: the length of the string to write out
+ * @error: return location for a #GError, or %NULL
+ *
+ * Writes @len bytes of @buf to the file specified by @handle.
+ *
+ * Return value: %TRUE on success
+ **/
 gboolean
 totem_pl_parser_write_buffer (GnomeVFSHandle *handle, const char *buf, guint len, GError **error)
 {
@@ -442,6 +756,18 @@ totem_pl_parser_write_buffer (GnomeVFSHandle *handle, const char *buf, guint len
 	return TRUE;
 }
 
+/**
+ * totem_pl_parser_num_entries:
+ * @parser: a #TotemPlParser
+ * @model: a #GtkTreeModel
+ * @func: a pointer to a #TotemPlParserIterFunc callback function
+ * @user_data: a pointer to be passed to each call of @func
+ *
+ * Returns the number of entries in @parser's playlist, and calls
+ * @func for each valid entry in the playlist.
+ *
+ * Return value: the number of entries in the playlist
+ **/
 int
 totem_pl_parser_num_entries (TotemPlParser *parser, GtkTreeModel *model,
 			     TotemPlParserIterFunc func, gpointer user_data)
@@ -457,7 +783,7 @@ totem_pl_parser_num_entries (TotemPlParser *parser, GtkTreeModel *model,
 		char *url, *title;
 		gboolean custom_title;
 
-		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i -1) == FALSE)
+		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
 			return i - ignored;
 
 		func (model, &iter, &url, &title, &custom_title, user_data);
@@ -471,6 +797,19 @@ totem_pl_parser_num_entries (TotemPlParser *parser, GtkTreeModel *model,
 	return num_entries - ignored;
 }
 
+/**
+ * totem_pl_parser_relative:
+ * @url: a URI
+ * @output: a base path and filename
+ *
+ * Returns the URI of @url relative to @output if possible, and %NULL
+ * if not.
+ *
+ * <emphasis>See totem_pl_parser_resolve_url() to convert from relative URLs
+ * to absolute URLs.</emphasis>
+ *
+ * Return value: a newly-allocated relative URI string, or %NULL
+ **/
 char *
 totem_pl_parser_relative (const char *url, const char *output)
 {
@@ -517,6 +856,27 @@ totem_pl_parser_relative (const char *url, const char *output)
 }
 
 #ifndef TOTEM_PL_PARSER_MINI
+/**
+ * totem_pl_parser_write_with_title:
+ * @parser: a #TotemPlParser
+ * @model: a #GtkTreeModel
+ * @func: a pointer to a #TotemPlParserIterFunc callback function
+ * @output: the output path and filename
+ * @title: the playlist title
+ * @type: a #TotemPlParserType for the ouputted playlist
+ * @user_data: a pointer to be passed to each call of @func
+ * @error: return location for a #GError, or %NULL
+ *
+ * Writes the playlist held by @parser and @model out to the file of
+ * path @output. The playlist is written in the format @type and is
+ * given the title @title.
+ *
+ * For each entry in the @model, the function @func is called (and passed
+ * @user_data), which gets various metadata values about the entry for
+ * the playlist writer.
+ *
+ * Return value: %TRUE on success
+ **/
 gboolean
 totem_pl_parser_write_with_title (TotemPlParser *parser, GtkTreeModel *model,
 				  TotemPlParserIterFunc func,
@@ -547,6 +907,26 @@ totem_pl_parser_write_with_title (TotemPlParser *parser, GtkTreeModel *model,
 	return FALSE;
 }
 
+/**
+ * totem_pl_parser_write:
+ * @parser: a #TotemPlParser
+ * @model: a #GtkTreeModel
+ * @func: a pointer to a #TotemPlParserIterFunc callback function
+ * @output: the output path and filename
+ * @type: a #TotemPlParserType for the ouputted playlist
+ * @user_data: a pointer to be passed to each call of @func
+ * @error: return location for a #GError, or %NULL
+ *
+ * Writes the playlist held by @parser and @model out to the file of
+ * path @output. The playlist is written in the format @type and is given
+ * a %NULL title.
+ *
+ * For each entry in the @model, the function @func is called (and passed
+ * @user_data), which gets various metadata values about the entry for
+ * the playlist writer.
+ *
+ * Return value: %TRUE on success
+ **/
 gboolean
 totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
 		       TotemPlParserIterFunc func,
@@ -560,6 +940,16 @@ totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
 
 #endif /* TOTEM_PL_PARSER_MINI */
 
+/**
+ * totem_pl_parser_read_ini_line_int:
+ * @lines: a NULL-terminated array of INI lines to read
+ * @key: the key to match
+ *
+ * Returns the first integer value case-insensitively matching the specified
+ * key as an integer. The parser ignores leading whitespace on lines.
+ *
+ * Return value: the integer value, or -1 on error
+ **/
 int
 totem_pl_parser_read_ini_line_int (char **lines, const char *key)
 {
@@ -592,6 +982,19 @@ totem_pl_parser_read_ini_line_int (char **lines, const char *key)
 	return retval;
 }
 
+/**
+ * totem_pl_parser_read_ini_line_string_with_sep:
+ * @lines: a NULL-terminated array of INI lines to read
+ * @key: the key to match
+ * @dos_mode: %TRUE if the returned string should end in \r\0, instead of \n\0
+ * @sep: the key-value separator
+ *
+ * Returns the first string value case-insensitively matching the specified
+ * key, where the two are separated by @sep. The parser ignores leading whitespace
+ * on lines.
+ *
+ * Return value: a newly-allocated string value, or %NULL
+ **/
 char*
 totem_pl_parser_read_ini_line_string_with_sep (char **lines, const char *key,
 		gboolean dos_mode, const char *sep)
@@ -632,6 +1035,17 @@ totem_pl_parser_read_ini_line_string_with_sep (char **lines, const char *key,
 	return retval;
 }
 
+/**
+ * totem_pl_parser_read_ini_line_string:
+ * @lines: a NULL-terminated array of INI lines to read
+ * @key: the key to match
+ * @dos_mode: %TRUE if the returned string should end in \r\0, instead of \n\0
+ *
+ * Returns the first string value case-insensitively matching the
+ * specified key. The parser ignores leading whitespace on lines.
+ *
+ * Return value: a newly-allocated string value, or %NULL
+ **/
 char*
 totem_pl_parser_read_ini_line_string (char **lines, const char *key, gboolean dos_mode)
 {
@@ -641,98 +1055,7 @@ totem_pl_parser_read_ini_line_string (char **lines, const char *key, gboolean do
 static void
 totem_pl_parser_init (TotemPlParser *parser)
 {
-	GParamSpec *pspec;
 	parser->priv = G_TYPE_INSTANCE_GET_PRIVATE (parser, TOTEM_TYPE_PL_PARSER, TotemPlParserPrivate);
-
-	parser->priv->pspec_pool = g_param_spec_pool_new (FALSE);
-	pspec = g_param_spec_string ("url", "url",
-				     "URL to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("title", "title",
-				     "Title of the item to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("author", "author",
-				     "Author of the item to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("genre", "genre",
-				     "Genre of the item to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("base", "base",
-				     "Base URL of the item to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("volume", "volume",
-				     "Default playback volume (in percents)", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("autoplay", "autoplay",
-				     "Whether or not to autoplay the stream", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("duration", "duration",
-				     "String representing the duration of the entry, used for still images", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("starttime", "starttime",
-				     "String representing the start time of the stream (initial seek)", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("copyright", "copyright",
-				     "Copyright of the item to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("abstract", "abstract",
-				     "Abstract of the item to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("moreinfo", "moreinfo",
-				     "URL to get more information for item to be added", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("screensize", "screensize",
-				     "String representing the default movie size (double, full or original)", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("ui-mode", "ui-mode",
-				     "String representing the default UI mode (only compact is supported)", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("endtime", "endtime",
-				     "String representing the end time of the stream", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_boolean ("is-playlist", "is-playlist",
-				      "Boolean saying whether the entry pushed is the top-level of a playlist", FALSE,
-				      G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("description", "description",
-				     "String representing the description of the stream", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("publication-date", "publication-date",
-				     "String representing the publication date of the stream", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("filesize", "filesize",
-				     "String representing the filesize of a file", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("language", "language",
-				     "String representing the language of a stream", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("contact", "contact",
-				     "String representing the contact for a playlist", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("image-url", "image-url",
-				     "String representing the location of an image for a playlist", NULL,
-				     G_PARAM_READABLE & G_PARAM_WRITABLE);
-	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
@@ -776,7 +1099,7 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 		char *error = NULL;
 		const char *string;
 
-		pspec = g_param_spec_pool_lookup (parser->priv->pspec_pool,
+		pspec = g_param_spec_pool_lookup (totem_pl_parser_pspec_pool,
 						  name,
 						  G_OBJECT_TYPE (parser),
 						  FALSE);
@@ -849,6 +1172,16 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 	g_object_unref (G_OBJECT (parser));
 }
 
+/**
+ * totem_pl_parser_add_url:
+ * @parser: a #TotemPlParser
+ * @first_property_name: the first property name
+ * @Varargs: value for the first property, followed optionally by more
+ * name/value pairs, followed by %NULL
+ *
+ * Adds a URL to the playlist with the properties given in @first_property_name
+ * and @Varargs.
+ **/
 void
 totem_pl_parser_add_url (TotemPlParser *parser,
 			 const char *first_property_name,
@@ -860,6 +1193,14 @@ totem_pl_parser_add_url (TotemPlParser *parser,
 	va_end (var_args);
 }
 
+/**
+ * totem_pl_parser_add_one_url:
+ * @parser: a #TotemPlParser
+ * @url: the entry's URL
+ * @title: the entry's title
+ *
+ * Adds a single URL entry with only URL and title strings to the playlist.
+ **/
 void
 totem_pl_parser_add_one_url (TotemPlParser *parser, const char *url, const char *title)
 {
@@ -897,6 +1238,19 @@ totem_pl_parser_might_be_file (const char *url)
 	return TRUE;
 }
 
+/**
+ * totem_pl_parser_resolve_url:
+ * @base: a base path and filename
+ * @url: a URI
+ *
+ * Returns the absolute URI of @url, resolving any relative
+ * paths with respect to @base.
+ *
+ * <emphasis>See totem_pl_parser_relative() to convert from absolute URLs
+ * to relative URLs.</emphasis>
+ *
+ * Return value: a newly-allocated resolved URL
+ **/
 char *
 totem_pl_parser_resolve_url (const char *base, const char *url)
 {
@@ -1014,6 +1368,16 @@ static PlaylistTypes ignore_types[] = {
 	PLAYLIST_TYPE3 ("application/x-trash"),
 };
 
+/**
+ * totem_pl_parser_scheme_is_ignored:
+ * @parser: a #TotemPlParser
+ * @url: a URL
+ *
+ * Checks to see if @url's scheme is in the @parser's list of
+ * schemes to ignore.
+ *
+ * Return value: %TRUE if @url's scheme is ignored
+ **/
 gboolean
 totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, const char *url)
 {
@@ -1053,6 +1417,23 @@ totem_pl_parser_mimetype_is_ignored (TotemPlParser *parser,
 
 }
 
+/**
+ * totem_pl_parser_ignore:
+ * @parser: a #TotemPlParser
+ * @url: a URL
+ *
+ * Checks if the URL should be ignored. URLs are <emphasis>not</emphasis> ignored if:
+ * <itemizedlist>
+ *  <listitem><para>they have an unknown mimetype,</para></listitem>
+ *  <listitem><para>they have a special mimetype,</para></listitem>
+ *  <listitem><para>they have a mimetype which could be a video or a playlist.</para></listitem>
+ * </itemizedlist>
+ *
+ * URLs are automatically ignored if their scheme is ignored as per totem_pl_parser_scheme_is_ignored(),
+ * and are ignored if all the other tests are inconclusive.
+ *
+ * Return value: %TRUE if @url is to be ignored
+ **/
 gboolean
 totem_pl_parser_ignore (TotemPlParser *parser, const char *url)
 {
@@ -1238,6 +1619,19 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url,
 	return ret;
 }
 
+/**
+ * totem_pl_parser_parse_with_base:
+ * @parser: a #TotemPlParser
+ * @url: the URL of the playlist to parse
+ * @base: the base path for relative filenames
+ * @fallback: %TRUE if the parser should add the playlist URL to the
+ * end of the playlist on parse failure
+ *
+ * Parses a playlist given by the absolute URL @url, using
+ * @base to resolve relative paths where appropriate.
+ *
+ * Return value: a #TotemPlParserResult
+ **/
 TotemPlParserResult
 totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *url,
 				 const char *base, gboolean fallback)
@@ -1256,6 +1650,17 @@ totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *url,
 	return totem_pl_parser_parse_internal (parser, url, base);
 }
 
+/**
+ * totem_pl_parser_parse:
+ * @parser: a #TotemPlParser
+ * @url: the URL of the playlist to parse
+ * @fallback: %TRUE if the parser should add the playlist URL to the
+ * end of the playlist on parse failure
+ *
+ * Parses a playlist given by the absolute URL @url.
+ *
+ * Return value: a #TotemPlParserResult
+ **/
 TotemPlParserResult
 totem_pl_parser_parse (TotemPlParser *parser, const char *url,
 		       gboolean fallback)
@@ -1263,6 +1668,14 @@ totem_pl_parser_parse (TotemPlParser *parser, const char *url,
 	return totem_pl_parser_parse_with_base (parser, url, NULL, fallback);
 }
 
+/**
+ * totem_pl_parser_add_ignored_scheme:
+ * @parser: a #TotemPlParser
+ * @scheme: the scheme to ignore
+ *
+ * Adds a scheme to the list of schemes to ignore, so that
+ * any URL using that scheme is ignored during playlist parsing.
+ **/
 void
 totem_pl_parser_add_ignored_scheme (TotemPlParser *parser,
 		const char *scheme)
@@ -1273,6 +1686,14 @@ totem_pl_parser_add_ignored_scheme (TotemPlParser *parser,
 		(parser->priv->ignore_schemes, g_strdup (scheme));
 }
 
+/**
+ * totem_pl_parser_add_ignored_mimetype:
+ * @parser: a #TotemPlParser
+ * @mimetype: the mimetype to ignore
+ *
+ * Adds a mimetype to the list of mimetypes to ignore, so that
+ * any URL of that mimetype is ignored during playlist parsing.
+ **/
 void
 totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
 		const char *mimetype)
@@ -1283,6 +1704,16 @@ totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
 		(parser->priv->ignore_mimetypes, g_strdup (mimetype));
 }
 
+/**
+ * totem_pl_parser_parse_duration:
+ * @duration: the duration string to parse
+ * @debug: %TRUE if debug statements should be printed
+ *
+ * Parses the given duration string and returns it as a <type>gint64</type>
+ * denoting the duration in seconds.
+ *
+ * Return value: the duration in seconds, or -1 on error
+ **/
 gint64
 totem_pl_parser_parse_duration (const char *duration, gboolean debug)
 {
@@ -1348,6 +1779,16 @@ totem_pl_parser_is_iso8601_date (const char *date_str)
 	return TRUE;
 }
 
+/**
+ * totem_pl_parser_parse_date:
+ * @date_str: the date string to parse
+ * @debug: %TRUE if debug statements should be printed
+ *
+ * Parses the given date string and returns it as a <type>gint64</type>
+ * denoting the date in seconds since the UNIX Epoch.
+ *
+ * Return value: the date in seconds, or -1 on error
+ **/
 guint64
 totem_pl_parser_parse_date (const char *date_str, gboolean debug)
 {
@@ -1370,6 +1811,17 @@ totem_pl_parser_parse_date (const char *date_str, gboolean debug)
 
 #endif /* !TOTEM_PL_PARSER_MINI */
 
+/**
+ * totem_pl_parser_can_parse_from_data:
+ * @data: the data to check for parsability
+ * @len: the length of data to check
+ * @debug: %TRUE if debug statements should be printed
+ *
+ * Checks if the first @len bytes of @data can be parsed, using the same checks
+ * and precedences as totem_pl_parser_ignore().
+ *
+ * Return value: %TRUE if @data can be parsed
+ **/
 gboolean
 totem_pl_parser_can_parse_from_data (const char *data,
 				     gsize len,
@@ -1413,6 +1865,16 @@ totem_pl_parser_can_parse_from_data (const char *data,
 	return FALSE;
 }
 
+/**
+ * totem_pl_parser_can_parse_from_filename:
+ * @filename: the file to check for parsability
+ * @debug: %TRUE if debug statements should be printed
+ *
+ * Checks if the file can be parsed, using the same checks and precedences
+ * as totem_pl_parser_ignore().
+ *
+ * Return value: %TRUE if @filename can be parsed
+ **/
 gboolean
 totem_pl_parser_can_parse_from_filename (const char *filename, gboolean debug)
 {
